////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * The context object, holds the context for a given editor.
 */
function Context(id, xopus)
{
  this.id = id;
  this.xopus = xopus;
  
  this.shouldRedraw = false;
}

Context.prototype.getEditor = function() {
  return this.xopus.htmldocument.getElementById(this.id);
};

Context.prototype.toString = function()
{
  var node = this.getEditorElement();
  
  if (!node)
    return "#deleted";
  
  if (this.getEditor().editable && this.getEditor().getAttribute("xopus_type") == "element") 
    node = node.parentNode;
  
  return this.getPrettyName(node);
};

Context.prototype.hasChanged = function()
{
  return xopus_globs.editorDriver.hasChanged(this.getEditor(), this);
};

Context.prototype.getDOMRange = function()
{
  var editor = this.getEditor();
  var editorNode = this.getEditorNode();
  var rng = this.xopus.htmldocument.selection.createRange();
  
  // See if the current selection is contained in the current editor.
  if (rng.length || !xopus_globs.utils.isAncestorOf(editor, rng.parentElement()))
    return null;
  
//alert(editor.childNodes[0].outerHTML);
  
  // Map the current selection to XML
  return this.xopus.convertRangeToXML(editor.childNodes[0], editorNode, rng);
};

Context.prototype.getSelectionNode = function()
{
  // Use selection only if the editor is editable
  if (!this.getEditor().editable)
    return this.getEditorNode();

  var domRange = this.getDOMRange();

  if (!domRange)
    return this.getEditorNode();
  else
    return domRange.commonAncestorContainer;
};

Context.prototype.getSelectionElement = function()
{
  var node = this.getSelectionNode();
  
  if (node.nodeType == 3)
    return node.parentNode;
  else
    return node;
};

Context.prototype.getEditorNode = function()
{
  var xpath = this.getEditor().getAttribute("xpath");
  var node = this.xopus.getNode(xpath);
  
  if (!node)
    return null;
  
  // This could be a fake node.
  if (!node.specified)
    node = this.xopus.getNode(xpath + "/parent::*");

  return node;
};

Context.prototype.getEditorElement = function()
{
  var node = this.getEditorNode();
  
  if (!node)
    return null;
  
  if (node.nodeType == 3)
    node = node.parentNode;
  
  return node;
};

Context.prototype.getAncestors = function()
{
  return this.getSelectionNode().selectNodes("ancestor-or-self::*");
};

Context.prototype.getSchemaDeclaration = function(node)
{
  if (!node)
    __xopus_alert("Context.prototype.getSchemaDeclaration called with null argument.");
  
  return this.xopus.xmlhandler.schemahandler.getDeclaration(node);
};

Context.prototype.getSchemaAnnotation = function(node)
{
  if (!xopus_globs.useXopusSOM)
  {
    // TODO This doesn't seem to be possible when using Microsoft SOM 
    return null;
  }
  else
  {
    if (!node)
      __xopus_alert("Context.prototype.getSchemaAnnotation called with null argument.");
    
    var decl = this.getSchemaDeclaration(node);
    
    try {
      return decl.annotation;
    } catch(e) {
      return false;
    }
  }
};

Context.prototype.getPrettyName = function(node)
{
  var ann = this.getSchemaAnnotation(node);
  var name = xopus_globs.locale.getString("pretty:" + node.nodeName, {});
  
  return this.getPrettyStringFromXML(ann, "name", name);
};

Context.prototype.getPrettyNameFromDeclaration = function(decl)
{
  var ann = decl.annotation;
  var name = xopus_globs.locale.getString("pretty:" + decl.name, {});
  
  return this.getPrettyStringFromXML(ann, "name", name);
};

Context.prototype.getPrettyShortDescription = function(node)
{
  var desc = null;
  for (var n = node; !desc && n && n.nodeType != 9; n = n.parentNode)
  {
    var ann = this.getSchemaAnnotation(n);
    desc = this.getPrettyStringFromXML(ann, "shortDescription", null);
  }
  
  if (!desc)
    desc = this.getPrettyName(node);
  
  return desc;
};

Context.prototype.getPrettyShortDescriptionFromDeclaration = function(decl)
{
  var ann = decl.annotation;
  var name = this.getPrettyNameFromDeclaration(decl);
  
  return this.getPrettyStringFromXML(ann, "shortDescription", name);
};

Context.prototype.getPrettyLongDescription = function(node)
{
  var desc = null;
  for (var n = node; !desc && n && n.nodeType != 9; n = n.parentNode)
  {
    var ann = this.getSchemaAnnotation(n);
    desc = this.getPrettyStringFromXML(ann, "longDescription", null);
  }
  
  if (!desc)
    desc = this.getPrettyShortDescription(node);
  
  return desc;
};

Context.prototype.getPrettyLongDescriptionFromDeclaration = function(decl)
{
  var ann = decl.annotation;
  var fallback = this.getPrettyShortDescriptionFromDeclaration(decl);
  
  return this.getPrettyStringFromXML(ann, "longDescription", fallback);
};

Context.prototype.getPrettyStringFromXML = function(xml, name, fallback)
{
  if (xml)
  {
    var doc = xml.selectSingleNode("*[local-name()='appinfo' and namespace-uri()='http://www.w3.org/2001/XMLSchema']" +
                                   "/*[local-name()='documentation' and namespace-uri()='http://www.xopus.org/xmlns/xsd']");

    if (doc)
    {
      var nameNode = null;
      var langs = xopus_globs.locale.getLanguages();

      for (var i = 0; !nameNode && i < langs.length; i++)
        nameNode = doc.selectSingleNode("*[local-name()='" + name +"' and namespace-uri()='http://www.xopus.org/xmlns/xsd']" +
                                        "[lang('" + langs[i] + "')]/node()");
      if (nameNode)
        return nameNode.xml;
    }
  }
  
  return fallback;
};

Context.prototype.canContainNode = function(parent, nodeName, namespaceURI)
{
  return this.xopus.xmlhandler.schemahandler.canContainNode(parent, nodeName, namespaceURI);
};

Context.prototype.getPossibleChildren = function(parent)
{
  return this.xopus.xmlhandler.schemahandler.getPossibleChildren(parent);
};

Context.prototype.createChildNode = function(parent, localname, namespaceURI)
{
  var decl = this.xopus.xmlhandler.schemahandler.getChildDeclaration(parent, localname, namespaceURI);
  
  if (!decl)
    xopus_globs.errorHandler.showFatalError("couldntFindSchemaDecl", {parent: parent.nodeName, child: localname, namespace: namespaceURI});
    
  return this.xopus.xmlhandler.createNode(decl, parent);
};

Context.prototype.canMoveUp = function(node)
{
  return this.xopus.xmlhandler.schemahandler.canMoveUp(node);
};

Context.prototype.canMoveDown = function(node)
{
  return this.xopus.xmlhandler.schemahandler.canMoveDown(node);
};

Context.prototype.canDelete = function(node)
{
  return this.xopus.xmlhandler.schemahandler.canDeleteNode(node);
};

/**
 * Returns a mapping of the names of different properties of the context to their values.
 */
Context.prototype.getHash = function()
{
  return {nodeName: this.getSelectionNode().nodeName};
};

Context.prototype.getXopus = function()
{
  return this.xopus;
};

Context.prototype.getHTMLDocument = function()
{
  return this.getXopus().htmldocument;
};

/**
 * Redraw control.
 */
Context.prototype.requestRedraw = function()
{
  this.shouldRedraw = true;
};


Context.prototype.redrawRequested = function()
{
  var res = this.shouldRedraw;
  this.shouldRedraw = false;
  
  return res;
};

/**
 * Change the current Context
 */
Context.changeTo = function(id, xopus) 
{
  // Reset all editors.
  xopus_globs.editorDriver.reset();
  
  Context.previousContext = Context.cachedContext;
  Context.currentID = id;
  Context.currentXopus = xopus;
  
  if (Context.hasCurrent())
    xopus_globs.toolbar.setStatusDefaultValue(Context.getCurrent().toString());
  else
    xopus_globs.toolbar.setStatusDefaultValue(null);
};

Context.lose = function() {
  this.changeTo(null, null);
};

Context.getContext = function()
{
  if (Context.hasCurrent())
    return Context.getCurrent();
  else
    return Context.getPrevious();
};

Context.getCurrent = function()
{
  if (Context.currentID == null)
    return null;
  
  // Validate current cache.
  var editor = this.currentXopus.htmldocument.getElementById(Context.currentID);
  
  if (!(Context.cachedContext && Context.cachedContext.getEditor() == editor))
    Context.cachedContext = new Context(Context.currentID, Context.currentXopus);
  return Context.cachedContext;
};

Context.hasCurrent = function()
{
  if (!Context.currentID)
    return false;
  
  if (!Context.currentXopus || !Context.currentXopus.htmldocument.getElementById(Context.currentID)) {
    this.lose();
    return false;
  }
  
  return true;
};

Context.getPrevious = function()
{
  if (!Context.previousContext || !Context.previousContext.xopus || !Context.previousContext.xopus.htmldocument) return null;
  return Context.previousContext;
};

Context.hasPrevious = function()
{
  return Context.previousContext != null;
};

Context.applyLanguage = function()
{
  if (Context.hasCurrent())
    xopus_globs.toolbar.setStatusDefaultValue(Context.getCurrent().toString());
  else
    xopus_globs.toolbar.setStatusDefaultValue(null);
};

numScriptsLoading--;
