////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * Create a new EditorDriver.
 */
function EditorDriver()
{
  this.editorTools = new EditorTools();
  this.initializeEditorTypes(this.editorTools);
  this.storage = {};
}

/**
 * All functions in this object have the following parameters:
 *
 * editor is a DHTML DIV object, contains the HTML being edited. Usually this is only a text node, but it can be a whole
 *
 * context is an object with the following interface:
 *
 *   getEditor()
 *
 *     Returns the current editor. Equals the editor parameter.  
 *
 *   getCurrentNode()
 *
 *     Returns the current node. If the current node is a text node, its parent will be returned.
 *
 *   getAncestors()
 *
 *     Returns a node list with all ancestors of the current node and the current node. 
 *
 *   getPossibleChildren()
 *
 *     Returns a local name, schema declaration mapping for all possible children for the current node.
 *
 *   getSchemaDeclaration()
 *
 *     Returns the schema declaration for the current node. 
 *     The interface is equal to the ISchemaElement or ISchemaAttribute MSXML4 SOM interface.
 *
 *   canContainNode(nodeName, namespaceURI)
 *
 *     nodeName:      the local name of the element being tested for insertion.
 *     namespaceURI:  (optional) the namespaceURI for the element being tested for insertion.
 *
 *     Returns true if the current node can contain the specified node.
 *
 *   getHash()
 *
 *     Returns an object with name value mappings describing the current context.
 *     Contains:
 *
 *       nodeName
 */
EditorDriver.prototype =
{
  /**
   * The specified editor is created. This is called after every redraw. The context can be null.
   */
  create: function (editor, context)
  {
    this.selectEditorType(editor, context).create(editor, context, this.getStorage(editor));
  },
  
  /**
   * The specified editor is destroyed. This is called before every redraw. The context can be null.
   */
  destroy: function (editor, context)
  {
    this.selectEditorType(editor, context).destroy(editor, context, this.getStorage(editor));
  },  
  
  /**
   * The mouse cursor hovers over the specified editor.
   */
  mouseOver: function (editor, context)
  {
    this.selectEditorType(editor, context).mouseOver(editor, context, this.getStorage(editor));
  },
  
  /**
   * The mouse cursor left the specified editor.
   */
  mouseOut: function (editor, context)
  {
    this.selectEditorType(editor, context).mouseOut(editor, context, this.getStorage(editor));
  },
  
  /**
   * Start the specified editor. The editor got focus.
   */
  start: function (editor, context)
  {
    this.selectEditorType(editor, context).start(editor, context, this.getStorage(editor));
  },
  
  /**
   * Stop the specified editor. The editor lost focus.
   */
  stop: function (editor, context)
  {
    this.selectEditorType(editor, context).stop(editor, context, this.getStorage(editor));
  },
  
  /**
   * Update the content for the specified editor. The user is typing in another editor containing the same node.
   * This event is fired to synchronize the two editors.
   */
  update: function (editor, context, newHTML)
  {
    this.selectEditorType(editor, context).update(editor, context, newHTML, this.getStorage(editor));
  },
  
  /**
   * An editor is started that contains the same node as this editor.
   */
  sync: function (editor, context)
  {
    this.selectEditorType(editor, context).sync(editor, context, this.getStorage(editor));
  },
  
  /**
   * An editor is stopped that contains the same node as this editor.
   */
  unsync: function (editor, context)
  {
    this.selectEditorType(editor, context).unsync(editor, context, this.getStorage(editor));
  },
  
  /**
   * An error occured in the specified editor.
   */
  error: function (editor, context)
  {
    this.selectEditorType(editor, context).error(editor, context, this.getStorage(editor));
  },
  
  /**
   * Asks if the speicfied editor has changed.
   */
  hasChanged: function (editor, context)
  {
    return this.selectEditorType(editor, context).hasChanged(editor, context, this.getStorage(editor));
  },
  
  /**
   * Select the editor type for the given editor and context.
   */
  selectEditorType: function(editor, context)
  {
    // Check cache first.
    var xpath = editor.getAttribute("xpath");
    var cachedType = context.xopus.cache.get(xpath, "0");
    if (cachedType != null)
      return cachedType;
    
    for (var i = 0; i < this.editorTypes.length; i++)
    {
      if (this.editorTypes[i].accept(editor, context, this.getStorage(editor)))
      {
        // Store in cache
        context.xopus.cache.put(xpath, "0", this.editorTypes[i]);
        
        return this.editorTypes[i];
      }
    }
  },
  
  /**
   * Get storage for the given editor, or create new storage if none existed.
   */
  getStorage: function(editor)
  {
    if (!this.storage[editor.id])
      this.storage[editor.id] = {};
      
    return this.storage[editor.id];
  },
  
  /**
   * Reset all editors.
   */
  reset: function()
  {
    this.storage = {};
  }
};

/**
 * Contains some editor helper functions.
 */
function EditorTools(){}

EditorTools.prototype = 
{
  curAttentionElement: null,
  curEditingElement:   null,
  attentionTOH: null,
  
  hideBorder: function (theEl)
  {
    if (xopus_globs.doAnimations) 
    {
      try {
        var targetColor = xopus_globs.animator.getValue(theEl, "backgroundColor").s;
      } catch(e) {
        var targetColor = "#FFFFFF";
      }
    
      setTimeout(function() {
        xopus_globs.animator.start({
          els:theEl, 
          anims:
          {
            prop:"borderColor", 
            target:targetColor,
            duration:500,
            profile:xopus_globs.animator.LINEAR,
            endCode:'el.className = "xopus_noborder"'
          }
        });
      }, 1);
    }
    else 
    {
      theEl.className = "xopus_noborder";
    }
  },
  
  showBorder: function (theEl, className)
  {
    theEl.className = className;
    if (xopus_globs.doAnimations) 
    {
      setTimeout(function() {
        xopus_globs.animator.abort([theEl],["borderColor"]);
        theEl.style.borderColor = "";
      }, 1);
    }
  },
  
  showAttentionBorder: function (editor)
  {
    if (this.curAttentionElement) 
      this.hideAttentionBorder(this.curAttentionElement, true);
    
    this.curAttentionElement = editor;
    this.showBorder(editor, "xopus_attentionborder");

    // Mozilla workaround
    // Mozilla does not detect mouseout events properly
    // therefore we will hide attention borders automatically
    if (xopus_globs.browser.moz) {
      if (this.attentionTOH) clearTimeout(this.attentionTOH);
      var myself = this;
      this.attentionTOH = setTimeout(function(){myself.hideAttentionBorder();}, 2000);
    }
  },
  
  hideAttentionBorder: function (editor, noAnim)
  {
    if (!this.curAttentionElement)
      return;
    
    if (!noAnim)
      this.hideBorder(this.curAttentionElement);
    this.curAttentionElement = null;
  },
  
  showEditingBorder: function (editor) 
  {
    if (this.curEditingElement) 
      this.hideEditingBorder(this.curEditingElement, true);
    
    if (this.curAttentionElement) 
      this.hideAttentionBorder(this.curAttentionElement, true);
    
    this.curEditingElement = editor;
    this.showBorder(editor, "xopus_editborder");
  },
  
  hideEditingBorder: function (editor, noAnim)
  {
    if (!this.curEditingElement)
      return;

    if (!noAnim)
      this.hideBorder(this.curEditingElement);
    this.curEditingElement = null;
  },
  
  showSyncedBorder: function (editor)
  {
    this.showBorder(editor, "xopus_syncborder");
  },
  
  hideSyncedBorder: function (editor)
  {
    this.hideBorder(editor);
  },
  
  showDefaultBorder: function (editor)
  {
    editor.className = "xopus_noborder";
  },
  
  hideDefaultBorder: function (editor)
  {
    this.hideBorder(editor);
  },
  
  showErrorBorder: function (editor)
  {
    this.showBorder(editor, "xopus_errorborder");
  },
  
  hideErrorBorder: function (editor)
  {
    this.hideBorder(editor);
  }
};

numScriptsLoading--;
