////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * Create a new pipeline from the given pipeline definition.
 */
function Pipeline(pipelineDefinition, driver)
{
  this.driver           = driver;
  this.views            = pipelineDefinition.views;
  this.defaultView      = pipelineDefinition.defaultView;
  this.xmlURI           = pipelineDefinition.xmlURI;
  this.addPaths         = this.loadXopusTemplate("pipeline_add_paths.xsl");
  this.addXIncludePaths = this.loadXopusTemplate("pipeline_add_xinclude_paths.xsl");
  this.keepIds          = this.loadXopusTemplate("pipeline_keepids.xsl");
  this.finalize         = this.loadXopusTemplate("pipeline_finalize.xsl");
  
  this.xopusId = 0;
  this.xIncludePaths = {};
    
  this.prepare();
}

Pipeline.prototype=
{
  /**
   * Prepare all views.
   */
  prepare: function()
  {
    for (var view in this.views)
      this.prepareView(this.views[view]);
  },
  
  /**
   * Prepare all xsl stylesheets in the given view for use with Xopus.
   */
  prepareView: function(view)
  {
    var lastXSLFound = false;

    // Iterate through all actions in the pipeline.
    for (var i = view.actions.length-1; i>=0; i--)
    {
      var action = view.actions[i];
      
      // Only prepare transforms.
      if (action.type == "transform") {
        action.xsl = this.prepareTemplate(action.cleanXsl, this.keepIds, xopus_globs.utils.getPath(action.uri));

        if (!lastXSLFound) {
          //ensures single rooted output of this XSL
          this.correctLastXSL(action.xsl);

          lastXSLFound = true;
        }
      }
    }
  },
  correctLastXSL:function(lastXSL) {
    //ensures single rooted output of this XSL
    //try to make sure the xsl's output is single rooted
    
    //step 1: find the template having @match="/" and not having a @mode
//alert(lastXSL.xml);
    var tpl = lastXSL.selectSingleNode("/*[local-name()='stylesheet' and namespace-uri()='http://www.w3.org/1999/XSL/Transform']/*[local-name()='template' and namespace-uri()='http://www.w3.org/1999/XSL/Transform' and @match='/' and not(@mode)]");

    if (tpl) {
      //add a @name to this template
      tpl.setAttribute("name", "xopus_real_roottpl");

      //remove the @match from this template
      tpl.removeAttribute("match");
  
      //add a template to the bottom of the stylesheet, looking this way:
      //  <xsl:template match="/">
      //    <div>
      //      <xsl:call-template name="xopus_real_roottpl" />
      //    </div>
      //  </xsl:template>
      var xsltplNode = lastXSL.createNode(1, "xsl:template", "http://www.w3.org/1999/XSL/Transform");
      xsltplNode.setAttribute("match", "/");

      var divNode = lastXSL.createElement("div");

      var xslcalltplNode = lastXSL.createNode(1, "xsl:call-template", "http://www.w3.org/1999/XSL/Transform");
      xslcalltplNode.setAttribute("name", "xopus_real_roottpl");

      divNode.appendChild(xslcalltplNode);
      xsltplNode.appendChild(divNode);
      lastXSL.documentElement.appendChild(xsltplNode);
    }
  },
  
  /**
   * Prepare the given template with the given enrich xsl.
   */
  prepareTemplate: function (template, enrich, basePath)
  {
    // Resolve includes in original xsl
    this.resolveXSLIncludes(template, basePath);
    
    var prepared;
    if (xopus_globs.enrichXSL)
      prepared = xopus_globs.xmllib.transform(template, enrich);
    else
      prepared = template;
    
    // Resolve includes in enriched xsl
    this.resolveXSLIncludes(prepared, xopus_globs.basePath);
    
    return prepared;
  },
  
  /**
   * Resolves all xsl:include elements in the given xsl agains the given base path.
   */
  resolveXSLIncludes: function (template, basePath)
  {
    var includes = template.selectNodes("//*[local-name()='include' and namespace-uri()='http://www.w3.org/1999/XSL/Transform']");
    
    for (var i = includes.length - 1; i >= 0; i--)
    {
      // Load the included xsl.
      var ref = includes[i].getAttribute("href");
      var xopusInclude = includes[i].getAttribute("xopus");
      
      var newURI;
      var includedXsl;
      if (xopusInclude) {
        newURI = xopus_globs.utils.getAbsoluteURI(xopus_globs.basePath, ref);
        includedXsl = this.driver.loadXopusTemplate(newURI).xsl;
      } else {
        newURI = xopus_globs.utils.getAbsoluteURI(basePath, ref);
        includedXsl = this.driver.loadTemplate(newURI).xsl;
      }
      
      this.resolveXSLIncludes(includedXsl, newURI);
      
      // Find all xsl:template and variable elements.
      var xslTemplates = includedXsl.selectNodes("/*[local-name()='stylesheet' and namespace-uri()='http://www.w3.org/1999/XSL/Transform']/*[local-name()='template' or local-name()='variable' and namespace-uri()='http://www.w3.org/1999/XSL/Transform']");
      
      var rootElement = template.selectSingleNode("/*[local-name()='stylesheet' and namespace-uri()='http://www.w3.org/1999/XSL/Transform']");
      for (var j = 0; j < xslTemplates.length; j++)
      {
        var namespaceDeclarations = xslTemplates[j].selectNodes("namespace::node()");
        
        // Add namespace declarations for all namespaces and prefixes known in the template.
        for (var k = 0; k < namespaceDeclarations.length; k++)
          if (namespaceDeclarations[k].nodeValue != "http://www.w3.org/XML/1998/namespace")
            xslTemplates[j].setAttributeNode(namespaceDeclarations[k].cloneNode(true));
        
        // Copy the xsl:template element.
        rootElement.appendChild(xslTemplates[j].cloneNode(true));
      }
      
      // Remove the original include element.
      includes[i].parentNode.removeChild(includes[i]);
    }
  },
  
  /**
   * Resolves all xinclude elements in the given data.
   */
  resolveXIncludes: function (data, basePath)
  {
    // Store all xpaths to elements used as XIncludes
    this.xIncludePaths = {};
    
    // Find all xi:include elements that are not contained in an xi:include element.
    var includes = data.selectNodes("//*[local-name()='include' and namespace-uri()='http://www.w3.org/2001/XInclude' and " +
                                    "not(ancestor::*[local-name()='include' and namespace-uri()='http://www.w3.org/2001/XInclude'])]");
    
    for (var i = 0; i < includes.length; i++)
    {
      var include = includes[i];
      var href = include.selectSingleNode("@*[local-name()='href']");
      
      if (!href)
        xopus_globs.errorHandler.showFatalError("xincludeNoHref", {uri: basePath, xpath: xopus_globs.utils.getXPath(include)});
      
      href = href.nodeValue;
      
      // Find the marker
      var pi = include.selectSingleNode("following-sibling::processing-instruction('xopus_element')[substring-before(., '=') = 'include'][1]");
      
      // TODO: generate error if the processing instruction was not found
      
      var includePath = pi.nodeValue.substr("include=".length);
      this.xIncludePaths[includePath] = true;
      
      // Split the uri and xpointer parts.
      var hashPos = href.indexOf("#");
      var uri;
      var pointer;
      if (hashPos == -1)
      {
        uri = href;
      }
      else
      {
        uri = href.substring(0, hashPos);
        pointer = href.substring(hashPos + 1, href.length);
      }
      
      // Load the included XML
      var newURI = xopus_globs.utils.getAbsoluteURI(basePath, uri);
      var includedDOM = this.driver.loadInclude(newURI);
      var pointerRef = pointer.substring("xpointer(".length, pointer.length - ")".length);
      
      // Assumes the result is one element.
      var refElt = includedDOM.selectSingleNode(pointerRef);
      
      if (!refElt)
        __xopus_alert("!refElt:" + pointerRef + "\n\n" + includedDOM.xml);
      
      // Copy all attributes
      var attributes = include.attributes;
      for (var j = attributes.length - 1; j >= 0; j--)
      {
        var attr = attributes[j];
        
        if (xopus_globs.utils.getLocalName(attr) == "href")
        {
          var newAttr = refElt.ownerDocument.createNode(2, "xi:href", "http://www.w3.org/2001/XInclude");
          newAttr.nodeValue = attr.nodeValue;
          attr = newAttr;
        }
        else
          include.removeAttributeNode(attr);
        
        refElt.setAttributeNode(attr);
      }
      
      // Replace the include with the referenced node.
      include.parentNode.replaceChild(refElt, include);
      
      // Add paths to the referenced node.
      var doc = xopus_globs.xmllib.transform(refElt, this.addXIncludePaths);
      
      // Copy generated elements and processing instructions
      for (var k = 1; k < doc.childNodes.length; k++)
        if (doc.childNodes[k].nodeType == 1 || doc.childNodes[k].nodeType == 7)
          refElt.parentNode.insertBefore(doc.childNodes[k], refElt);
      
      // Remove the unenriched element.
      refElt.parentNode.removeChild(refElt);
    }
    
    return data;
  },
  
  /**
   * This function can be used to check whether an xpath is an xpath to an element used as XInclude.
   */
  isXInclude: function(xpath)
  {
    return !!this.xIncludePaths[xpath];
  },
  
  /**
   * Repair a possible damaged enriched node.
   */
  rerichData: function (xmlDOM)
  {
    var newDOM;
    newDOM = this.unrichData(xmlDOM);
    newDOM = this.enrichData(newDOM);
    
    var pi = this.getPI(xmlDOM);
    var parent = xmlDOM.parentNode;
    
    var newElt = newDOM.selectSingleNode("*");
    var newPi  = newDOM.selectSingleNode("processing-instruction()[name() = 'xopus_element']");
    
    if (xmlDOM.nodeType != 9)
    {
      parent.replaceChild(newPi, xmlDOM);
      parent.insertBefore(newElt, newPi);
      
      if (pi)
        parent.removeChild(pi);
    }
    else
      return newDOM;
  },
  
  /**
   * Enrich the given XML for use in the Xopus pipeline.
   */
  enrichData: function (xmlDOM)
  {
    xmlDOM = this.addIDs(xmlDOM);
    
    if (xmlDOM.nodeType == 11)
    {
      var res = xmlDOM.cloneNode(false);
      for (var i = 0; i < xmlDOM.childNodes.length; i++)
        res.appendChild(this.addPIs(xmlDOM.childNodes[i]));
        
      xmlDOM = res;
    }
    else
    {
      xmlDOM = this.addPIs(xmlDOM);
    }
    
    return xmlDOM;
  },
  
  /**
   * Remove data used to enrich the given XML for use in the Xopus pipeline.
   */
  unrichData: function (xmlDOM)
  {
    xmlDOM = this.removeIDs(xmlDOM);
    xmlDOM = this.removePIs(xmlDOM);
    
    return xmlDOM;
  },
  
  /**
   * Add xopus ids to elements in the given xml which don't have an id.
   */
  addIDs: function (xmlDOM)
  {
    var elements = xmlDOM.selectNodes("descendant-or-self::*[not(@*[local-name()='id' and namespace-uri()='http://www.xopus.org/xmlns/id'])]");
    
    for (var i = 0; i < elements.length; i++)
      xopus_globs.utils.setAttributeNS(elements[i], "http://www.xopus.org/xmlns/id", "xopus:id", this.xopusId++);
    
    return xmlDOM;
  },
  
  /** 
   * Add processing instructions.
   */
  addPIs: function (xmlDOM)
  {
    var xslRes = xopus_globs.xmllib.transform(xmlDOM, this.addPaths);
    var res = xopus_globs.utils.getChildren(xslRes.documentElement);
    
    // Return a document if xmlDOM is a document 
    if (xmlDOM.nodeType == 9)
    {
      var newDoc = xmlDOM.cloneNode(false);
      
      for (var i = res.childNodes.length - 1; i >= 0; i--)
        newDoc.insertBefore(res.childNodes[i], newDoc.firstChild);
        
      res = newDoc;
    }
    
    return res;
  },
  
  /**
   * Remove xopus ids from the given xml.
   */
  removeIDs: function (xmlDOM)
  {
    var newDOM = xmlDOM.cloneNode(true);
    var elements = newDOM.selectNodes("//*");
    
    for (var i = 0; i < elements.length; i++)
    {
      var attributes = elements[i].attributes;
    
      for (var j = 0; j < attributes.length; j++)
      {
        if (attributes[j].namespaceURI == "http://www.xopus.org/xmlns/id")
          elements[i].removeAttributeNode(attributes[j]);
      }
    }
    
    return newDOM;
  },
  
  /** 
   * Remove processing instructions.
   *
   * Use only with documents
   */
  removePIs: function (xmlDOM)
  {
    var newDOM = xmlDOM.cloneNode(true);
    var PIs = newDOM.selectNodes("//processing-instruction()[substring-before(name(), '_') = 'xopus']");
    
    for (var i = 0; i < PIs.length; i++)
    {
      var pi = PIs[i];
      
      pi.parentNode.removeChild(pi);
    }
    
    return newDOM;
  },
  
  /**
   * Find the processing instruction for the given node.
   */
  getPI: function (node)
  {
    var name = node.nodeName;
    
    switch (node.nodeType)
    {
      case 1:
        return node.selectSingleNode("following-sibling::processing-instruction('xopus_element')[substring-before(., '=') = '" + name + "'][1]");
      case 2:
        return node.selectSingleNode("../processing-instruction('xopus_attribute')[substring-before(., '=') = '" + name + "']");
      case 3:
        return node.selectSingleNode("following-sibling::processing-instruction('xopus_text')[1]");
    }
  },
  
  /**
   * Find the xpath to the given node as used in the pipeline.
   */
  getXPath: function (node)
  {
    var pi = this.getPI(node);
    
    if (!pi) {
      var parentNode = node.selectSingleNode("..");
      xopus_globs.utils.debug("!pi for: " + node.xml + "\n\nparent:" + (parentNode?parentNode.xml:'-no parent node-'));
    }

    var piVal = pi.nodeValue;
    
    switch (node.nodeType)
    {
      case 1:
      case 2:
        return piVal.substr(piVal.indexOf("=") + 1);
      case 3:
        return piVal;
    }
  },
  
  /**
   * Process the given xml document.
   */
  process: function (xmlDoc, viewID, params)
  {
    var view;
    if (!viewID)
      view = this.defaultView;
    else
      view = this.views[viewID];
    
    if (!view)
      xopus_globs.errorHandler.showFatalError("unknownView", {viewID: viewID});
    
    var currXML = xmlDoc;
    
    for (var i = 0; i < view.actions.length; i++)
    {
      var action = view.actions[i];
      
      switch(action.type)
      {
        case "transform":
          currXML = xopus_globs.xmllib.transform(currXML, action, params);
          //xopus_globs.utils.debug("Transformed: " + action.uri + ":\n\n" +  currXML.xml);
          break;
        case "resolveXIncludes":
          currXML = this.resolveXIncludes(currXML, this.xmlURI);
          //xopus_globs.utils.debug(currXML.xml);
          break;
      }
    }
    
    // Replace all processing instructions with HTML compatible taging.
    currXML = xopus_globs.xmllib.transform(currXML, this.finalize);
    //xopus_globs.utils.debug("Transformed: " + this.finalize.uri + ":\n\n" +  currXML.xml);

    return { xml:currXML };
  },
  XML2HTML:function(xml) {
    // Unescape the HTML
    var theHTML = xml;
    //theHTML = theHTML.replace(/&lt;/g, "<");
    //theHTML = theHTML.replace(/&gt;/g, ">");
    //theHTML = theHTML.replace(/&amp;/g, "&");
    //theHTML = theHTML.replace(/&quot;/g, "\"");
    //theHTML = theHTML.replace(/&acute;/g, "'");
    theHTML = theHTML.replace(/><\/(hr|img|br)>/g, "\/>");
    
    return theHTML;
  },
  
  /**
   * Load a template xopus needs in the pipeline.
   */
  loadXopusTemplate: function (name)
  {
    var transformer = this.driver.loadXopusTemplate(xopus_globs.basePath + "xopus/pipeline/" + name);
    this.resolveXSLIncludes(transformer.xsl, xopus_globs.basePath);
    
    return transformer;
  }
};

/**
 * Creates a new pipeline from pipeline definition in the given DOM
 */
function PipelineDefinition(defXML, driver, basePath) {
  this.pipelineXML = defXML.documentElement;
  this.xmlURI = xopus_globs.utils.getAbsoluteURI(basePath, this.pipelineXML.getAttribute("xml"));
  this.xsdURI = xopus_globs.utils.getAbsoluteURI(basePath, this.pipelineXML.getAttribute("xsd"));
  
  // Create all views.
  this.views = [];
  var viewNodes = this.pipelineXML.selectNodes("view");
  for (var i = 0; i < viewNodes.length; i++)
  {
    var viewNode = viewNodes[i];
    var viewID = viewNode.getAttribute("id");
    var viewName = viewNode.getAttribute("name");

    // Create a new view
    this.views[viewID] = { actions: [], id: viewID };
    if (viewName) this.views[viewID].name = viewName;
    
    // This is the default view
    if (viewNode.getAttribute("default") == "true")
      this.defaultView = this.views[viewID];
    
    // Make sure we have a defaultview
    if (!this.defaultView) 
      this.defaultView = this.views[viewID];
    
    // Add all actions for this view
    var actionNodes = viewNode.selectNodes("transform|resolveXIncludes|resolvexincludes");
    for (var j = 0; j < actionNodes.length; j++)
    {
      var actionNode = actionNodes[j];
      
      switch(actionNode.nodeName.toLowerCase())
      {
        case "transform":
          var absURI = xopus_globs.utils.getAbsoluteURI(basePath, actionNode.getAttribute("xsl"));
        
          this.views[viewID].actions[this.views[viewID].actions.length] = 
            {
              type: "transform",
              uri: absURI,
              cleanXsl: driver.loadTemplate(absURI).xsl
            };
          break;
        case "resolvexincludes":
          this.views[viewID].actions[this.views[viewID].actions.length] = 
            {
              type: "resolveXIncludes"
            };
          break;
      }
    }
    
    if (actionNodes.length == 0)
    {
      xopus_globs.errorHandler.showWarning("viewHasNoActions", {uri: defURI, viewID: viewID});
      
      // Remove the view
      delete this.views[viewID];
    }
  }
  
  if (viewNodes.length == 0)
    xopus_globs.errorHandler.showFatalError("definitionHasNoViews", {uri: defURI});
}

numScriptsLoading--;
