////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * The object handles changes to the XML.
 */
function XMLHandler(xopus, pipeline) 
{
  this.schemahandler = new SchemaHandler(this); 
  this.xopus         = xopus;
  this.pipeline      = pipeline;
}

XMLHandler.prototype=
{
  /**
   * Load a schema.
   */
  loadSchema: function (schemaURI)
  {
    this.schemahandler.loadSchema(schemaURI);
  },
  
  /**
   * Load the XML.
   */
  loadXML: function (xmlURI)
  {
    this.xmlURI = xmlURI;
    
    var xmlDOM2 = xopus_globs.datadriver.loadData(xmlURI);  
    var xmlDOM = xopus_globs.objectFactory.createXMLDOM();  
    this.schemahandler.attachTo(xmlDOM);
    xmlDOM.loadXML(xmlDOM2.xml);
    
    // Something is wrong with the xml
    if (xopus_globs.errorHandler.hasDOMError(xmlDOM))
    {
      var hash = xopus_globs.errorHandler.getDOMError(xmlDOM, xmlURI);
      xopus_globs.errorHandler.showRecoverableError("couldNotParseXML", hash);
            
      // Retry loading
      return this.loadXML(xmlURI);    
    }
    
    return xmlDOM;
  },
  
  /**
   * Add a node with the given declaration to the given parent.
   */
  appendNode: function (newChild, parentNode, test)
  {
    if (newChild.nodeType != 2)
    {
      newChild = this.pipeline.enrichData(newChild);
      parentNode.appendChild(newChild);
    } 
    else
    {
      // Clone this attribute because it might have an parent after trying to insert it.
      parentNode.setAttributeNode(newChild.cloneNode(true));
    }
    
    // Invalidate the cache.
    if (!test)
      this.invalidateCache(parentNode);
    
    return newChild;
  },
  
  /**
   * Insert a new node in parentNode after the refNode.
   */
  insertNode: function (newChild, parentNode, refNode, test)
  {
    var xpath;
    
    if (newChild.nodeType != 2)
    {
      newChild = this.pipeline.enrichData(newChild);
      
      // Skip a following processing instruction.
      if (refNode && refNode.nextSibling && refNode.nextSibling.nodeType == 7)
        refNode = refNode.nextSibling;
      
      if (!refNode)
        parentNode.insertBefore(newChild, parentNode.firstChild);
      else if (refNode != parentNode.lastChild)
        parentNode.insertBefore(newChild, refNode.nextSibling);
      else
        parentNode.appendChild(newChild);
    } 
    else
    {
      // Clone this attribute because it might have an parent after trying to insert it.
      parentNode.setAttributeNode(newChild.cloneNode(true));
    }
    
    // Invalidate the cache.
    if (!test)
      this.invalidateCache(parentNode);
    
    return true;
  },
  
  /**
   * Replace a node in the given parent node.
   */
  replaceNode: function (newChild, oldChild, parentNode)
  {
    parentNode.insertBefore(newChild, oldChild);
    parentNode.removeChild(oldChild);
    
    this.pipeline.rerichData(parentNode);
  },
  
  /**
   * Create a node with the given declaration.
   */
  createNode: function (decl, parentNode)
  {
    return this.schemahandler.createNodeFromDeclaration(decl, parentNode);
  },
  
  /**
   * Inserts the given XML in the given node or its first empty descendant.
   *
   * NOTE: A call to either insertNode or appendNode is expected after calling this function.
   *
   * TODO: test here if the found node can contain nodes or text.
   */
  insertXML: function (theNode, theXML)
  {
    while (theNode.nodeType == 1 && theNode.childNodes.length > 0)
      theNode = theNode.childNodes[0];
      
    if (theNode.nodeType == 1)
      theNode.appendChild(theXML);
    
    return true;
  },
  
  /**
   * Move the given node up. 
   */
  moveUp: function(theNode, test)
  {
    var parentNode = theNode.parentNode;
    var prevSibling = theNode.selectSingleNode("preceding-sibling::*[1]");

    if (!prevSibling)
      return false;
    
    var pi = this.pipeline.getPI(theNode);
    
    parentNode.removeChild(theNode);
    parentNode.removeChild(pi);
    parentNode.insertBefore(theNode, prevSibling);
    parentNode.insertBefore(pi, prevSibling);
    
    // Invalidate the cache.
    if (!test)
      this.invalidateCache(parentNode);
        
    return true;
  },

  /**
   * Move the given node down. 
   */
  moveDown: function(theNode, test)
  {
    var parentNode = theNode.parentNode;
    var nextSibling = theNode.selectSingleNode("following-sibling::*[1]");

    if (!nextSibling)
      return false;
    
    var pi = this.pipeline.getPI(nextSibling);
    
    parentNode.removeChild(nextSibling);
    parentNode.insertBefore(nextSibling, theNode);
    parentNode.removeChild(pi);
    parentNode.insertBefore(pi, theNode);
    
    // Invalidate the cache.
    if (!test)
      this.invalidateCache(parentNode);
        
    return true;
  },

  /**
   * Delete the given node. 
   */
  deleteNode: function(theNode, parentNode, test)
  {
    parentNode = parentNode || theNode.selectSingleNode("..");

    var pi = this.pipeline.getPI(theNode);
    
    if (theNode.nodeType == 2)
      parentNode.removeAttributeNode(theNode);
    else
      parentNode.removeChild(theNode);
    
    parentNode.removeChild(pi);
    
    // Invalidate the cache.
    if (!test)
      this.invalidateCache(parentNode);
        
    return true;
  },
  
  /**
   * This makes the given node valid.
   */
  makeValid: function (theNode)
  {
    // Only validate elements for now.
    if (theNode.nodeType != 1)
      return;
    
    // It might be valid already.
    if (this.schemahandler.validator.isValid(theNode))
      return;
      
    var context = Context.getContext();
    var decl = context.getSchemaDeclaration(theNode);    
    
    var parent = theNode.parentNode;
    var idx    = xopus_globs.utils.indexOf(theNode);
    this.schemahandler.fireEvent("onIsInvalid", decl, theNode);
    
    // Check if the result is valid.
    if (!this.schemahandler.validator.isValid(parent.childNodes[idx]))
      xopus_globs.utils.debug("The result of the make valid action was not valid:\n\n" + parent.childNodes[idx].xml);
    
    /*
    var textNode = theNode.ownerDocument.createTextNode(theNode.text);
    this.insertNode(textNode, theNode.parentNode, theNode);
    this.deleteNode(theNode.parentNode, theNode);
    
    // Find out which children are invalid
    var invalidChildren = [];
    var hasValidChildren = false;
    for (var i = 0; i < theNode.childNodes.length; i++)
    {
      var child = theNode.childNodes[i];
      
      if (child.nodeType != 1)
        continue;
      
      if (!this.schemahandler.validator.isValid(child))
        invalidChildren.push(child);
      else
        hasValidChildren = true;
    }
    
    // If all children are invalid, remove the current element.
    if (!hasValidChildren || invalidChildren.length == 0)
    {
      var pi = this.pipeline.getPI(theNode);
      
      theNode.parentNode.insertBefore(xopus_globs.utils.getChildren(theNode), theNode);
      theNode.parentNode.removeChild(pi);
      theNode.parentNode.removeChild(theNode);
    }
    
    // Make all invalid children valid.
    for (var i = 0; i < invalidChildren.length; i++)
      this.makeValid(invalidChildren[i]);
    */
  },
  
  /**
   * Invalidate the cache for the given node.
   *
   * An element is expected here.
   */
  invalidateCache: function (node, recur)
  {
    // Invalidate cache for parent.
    if (!recur)
    {
      var parentPath = this.pipeline.getXPath(node);
      this.xopus.cache.invalidate(parentPath);
    }
        
    // Invalidate cache for attributes (if any).
    if (node.nodeType == 1)
    {
      var attributes = node.attributes;
      for (var i = 0; i < attributes.length; i++)
      {
        var attribute = attributes[i];
        if (attribute.namespaceURI == "http://www.xopus.org/xmlns/id" || 
            attribute.namespaceURI == "http://www.w3.org/XML/1998/namespace" || 
            attribute.nodeName.indexOf("xmlns") == 0 ||
            attribute.nodeName.indexOf("xml:") == 0 ||
            attribute.prefix == "xml" ||
            attribute.nodeValue == "http://www.xopus.org/xmlns/id")
          continue;
        
        var attributePath = this.pipeline.getXPath(attribute);
        this.xopus.cache.invalidate(attributePath);
      }
    }
    
    // Invalidate cache for all children.
    var children = node.childNodes;
    for (var i = 0; i < children.length; i++)
    {
      var child = children[i];
      if (child.nodeType == 7)
        continue;
      
      var childPath = this.pipeline.getXPath(child);
      this.xopus.cache.invalidate(childPath);
      this.invalidateCache(child, true);
    }
  }
};

numScriptsLoading--;